/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2011
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * -------------------------------------------------------------------
 *
 * History
 *   Nov 23, 2005 (Kilian Thiel): created
 */
package org.knime.base.node.mine.mds.distances;

import org.knime.base.node.mine.mds.DataPoint;
import org.knime.core.data.DataRow;

/**
 *
 * @author Kilian Thiel, University of Konstanz
 */
public interface DistanceManager {

    /**
     * Returns the distance between the given <code>DataRow</code>s, row1 and
     * row2. The distance metric is up to the concrete implementation.
     *
     * @param row1 First <code>DataRow</code> to compute distance.
     * @param row2 Second <code>DataRow</code> to compute distance.
     * @return The distance between given <code>DataRow</code>s.
     */
    public double getDistance(DataRow row1, DataRow row2);

    /**
     * Returns the distance between the given <code>DataPoint</code>s, point1
     * and point2. The distance metric is up to the concrete implementation.
     *
     * @param point1 First <code>DataPoint</code> to compute distance.
     * @param point2 Second <code>DataPoint</code> to compute distance.
     * @return The distance between given <code>DataRow</code>s.
     */
    public double getDistance(DataPoint point1, DataPoint point2);

    /**
     * @return The type of the <code>DistanceManager</code>. See
     *         <code>DistanceManagerFactory</code> for valid types.
     *
     * @see org.knime.base.node.mine.mds.distances.DistanceManagerFactory
     *      #COS_DIST
     * @see org.knime.base.node.mine.mds.distances.DistanceManagerFactory
     *      #EUCLIDEAN_DIST
     * @see org.knime.base.node.mine.mds.distances.DistanceManagerFactory
     *      #MANHATTAN_DIST
     */
    public String getType();
}
