/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2011
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * -------------------------------------------------------------------
 *
 * History
 *   29.09.2006 (Fabian Dill): created
 */
package org.knime.base.node.viz.plotter.box;

import javax.swing.JCheckBox;

import org.knime.base.node.viz.plotter.columns.MultiColumnPlotterProperties;
import org.knime.base.node.viz.plotter.props.BoxPlotAppearanceTab;
import org.knime.core.data.DoubleValue;

/**
 * Tab to select whether to normalize the drawing or not.
 *
 * @author Fabian Dill, University of Konstanz
 */
public class BoxPlotterProperties extends MultiColumnPlotterProperties {
    private final BoxPlotAppearanceTab m_normalizeTab;

    /**
     *
     *
     */
    @SuppressWarnings("unchecked")
    public BoxPlotterProperties() {
        super(DoubleValue.class);
        m_normalizeTab = new BoxPlotAppearanceTab();
        addTab(m_normalizeTab.getDefaultName(), m_normalizeTab);
    }

    /**
     *
     * @return the checkbox to force normalized presentation.
     */
    public JCheckBox getNormalizeCheckBox() {
         return m_normalizeTab.getNormalizeCheckBox();
    }

    /**
     * @param enabled True to enable the checkbox, false to disable it.
     * @see javax.swing.AbstractButton#setEnabled(boolean)
     */
    public void setCheckboxEnabled(final boolean enabled) {
        m_normalizeTab.getNormalizeCheckBox().setEnabled(enabled);
    }

    /**
     * @param selected The initial value for the normalization checkbox.
     * @see javax.swing.AbstractButton#setSelected(boolean)
     */
    public void setCheckboxSelected(final boolean selected) {
        m_normalizeTab.getNormalizeCheckBox().setSelected(selected);
    }

    /**
     * @param text The tooltip text for the normalization checkbox.
     * @see javax.swing.JComponent#setToolTipText(java.lang.String)
     */
    public void setCheckboxToolTipText(final String text) {
        m_normalizeTab.getNormalizeCheckBox().setToolTipText(text);
    }
}
